<?php

/**
 * @file
 * Administrative functions for PHPMailer integration module.
 */

/**
 * Form builder for both the Mime Mail hook and our own menu callback.
 */
function phpmailer_settings_form() {
  if (!module_exists('mimemail') || !variable_get('mimemail_alter', 0)) {
    $form['smtp_on'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use PHPMailer to send e-mails'),
      '#default_value' => variable_get('smtp_on', 0),
      '#description' => t('When enabled, PHPMailer will be used for e-mail transport.'),
    );
  }
  else {
    $form['smtp_on'] = array(
      '#type' => 'value',
      '#value' => 0,
    );
  }

  $form['server']['smtp_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Primary SMTP server'),
    '#default_value' => variable_get('smtp_host', 'localhost'),
    '#description' => t('The host name or IP address of your primary SMTP server.  Use !gmail-smtp for Google Mail.', array('!gmail-smtp' => '<code>smtp.gmail.com</code>')),
    '#required' => TRUE,
  );
  $form['server']['smtp_hostbackup'] = array(
    '#type' => 'textfield',
    '#title' => t('Backup SMTP server'),
    '#default_value' => variable_get('smtp_hostbackup', ''),
    '#description' => t('Optional host name or IP address of a backup server, if the primary server fails.  You can override the default port by appending it to the host name separated with a colon.  Example: !hostname', array('!hostname' => '<code>localhost:465</code>')),
  );
  $form['server']['smtp_port'] = array(
    '#type' => 'textfield',
    '#title' => t('SMTP port'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => variable_get('smtp_port', '25'),
    '#description' => t('The standard SMTP port is 25, for Google Mail use 465.'),
    '#required' => TRUE,
  );
  $form['server']['smtp_protocol'] = array(
    '#type' => 'select',
    '#title' => t('Use secure protocol'),
    '#default_value' => variable_get('smtp_protocol', ''),
    '#options' => array('' => t('No'), 'ssl' => t('SSL'), 'tls' => t('TLS')),
    '#description' => t('Whether to use an encrypted connection to communicate with the SMTP server.  Google Mail requires %SSL.', array('%SSL' => 'SSL')),
  );
  if (!function_exists('openssl_open')) {
    $form['server']['smtp_protocol']['#default_value'] = '';
    $form['server']['smtp_protocol']['#disabled'] = TRUE;
    $form['server']['smtp_protocol']['#description'] .= ' ' . t('Note: This option has been disabled since your PHP installation does not seem to have support for OpenSSL.');
    variable_set('smtp_protocol', '');
  }

  $form['auth'] = array(
    '#type' => 'fieldset',
    '#title' => t('SMTP authentication'),
    '#description' => t('Leave blank if your SMTP server does not require authentication.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['auth']['smtp_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => variable_get('smtp_username', ''),
    '#description' => t('For Google Mail, enter your username including "@gmail.com".'),
  );
  $form['auth']['smtp_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#default_value' => variable_get('smtp_password', ''),
  );

  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced SMTP settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['smtp_keepalive'] = array(
    '#type' => 'checkbox',
    '#title' => t('Keep connection alive'),
    '#default_value' => variable_get('smtp_keepalive', 0),
    '#description' => t('Whether to reuse an existing connection during a request.  Improves performance when sending a lot of e-mails at once.'),
  );
  $form['advanced']['smtp_fromname'] = array(
    '#type' => 'textfield',
    '#title' => t('"From" name'),
    '#default_value' => variable_get('smtp_fromname', ''),
    '#description' => t('Enter a name that should appear as the sender for all messages.  If left blank the site name will be used instead: %sitename.', array('%sitename' => variable_get('site_name', 'Drupal'))),
  );
  $form['advanced']['smtp_debug'] = array(
    '#type' => 'select',
    '#title' => t('Debug level'),
    '#default_value' => variable_get('smtp_debug', 0),
    '#options' => array(0 => t('Disabled'), 1 => t('Errors only'), 2 => t('Server responses'), 4 => t('Full communication')),
    '#description' => t("Debug the communication with the SMTP server.  You normally shouldn't enable this unless you're trying to debug e-mail sending problems."),
  );

  // Send a test email message if an address has been entered.
  if ($test_address = variable_get('smtp_test_address', '')) {
    // Delete first to avoid unintended resending in case of an error.
    variable_del('smtp_test_address');
    drupal_mail('phpmailer-test', $test_address, t('PHPMailer test e-mail'), t('Your site is properly configured to send e-mails using the PHPMailer library.'));
    drupal_set_message(t('A test e-mail has been sent to %email.  You may want to <a href="@watchdog-url">check the logs</a> for any error messages.', array('%email' => $test_address, '@watchdog-url' => url('admin/logs/watchdog'))));
  }

  $form['test'] = array(
    '#type' => 'fieldset',
    '#title' => t('Test configuration'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['test']['smtp_test_address'] = array(
    '#type' => 'textfield',
    '#title' => t('Recipient'),
    '#default_value' => '',
    '#description' => t('Type in an address to have a test e-mail sent there.'),
  );
  if (module_exists('mimemail')) {
    $form['test']['preview'] = array(
      '#type' => 'item',
      '#title' => t('Preview'),
      '#value' => t('For theming of HTML mails sent via Mime Mail, you can <a href="@preview-url" target="_blank">preview an example mail</a>.', array('@preview-url' => url('phpmailer/preview'))),
    );
  }

  if (module_exists('mimemail')) {
    return $form;
  }

  $form['#submit']['phpmailer_settings_form_submit'] = array();
  return system_settings_form($form);
}

/**
 * Form validation function.
 *
 * @see phpmailer_settings_form()
 */
function phpmailer_settings_form_validate($form_id, $form_values, $form) {
  if ($form_values['smtp_on']) {
    if (intval($form_values['smtp_port']) == 0) {
      form_set_error('smtp_port', t('You need to enter a valid SMTP port number.'));
    }
  }
}

/**
 * Form submit function.
 *
 * @see phpmailer_settings_form()
 */
function phpmailer_settings_form_submit($form_id, $form_values) {
  if ($form_values['smtp_on']) {
    if (!phpmailer_enabled()) {
      variable_set('smtp_library', drupal_get_filename('module', 'phpmailer'));
    }
  }
  else {
    if (phpmailer_enabled()) {
      variable_del('smtp_library');
    }
  }
}

